function [genes, I_transcript, cover] = median_transcript_intensities(I, probe_id, probe_label, genes, LABEL)

% [genes, I_transcript, cover] = median_transcript_intensities(I, 
%                                probe_id, probe_label, genes, LABEL)
%
% Computes median intensities across all probes complementary to the same
% transcript.
%
% I -- a vector of hybridization intensities of the same size as probe_id
% probe_id -- genomic locations (chr*10^8+position) of each probe
% probe_label -- a global annotation label for each probe (should be ==
%   LABEL.ambiguous in case of e.g. overlapping genes)
% genes -- a struct array with annotated genes
% LABEL -- a struct with all possible probe label constants
% returns a gene struct augmented by a hyb field (a struct with
%   gene-specific hybridization values), the ideal transcript intensity
%   for each probe and the coverage of each probe by annotated exons
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

VERBOSE = 0;
  
fprintf('Computing transcript intensity medians...\n');
I_transcript = -ones(size(probe_id));

tmp = [genes.transcripts];
tmp = length(tmp);
exon_means = zeros(1,tmp);
intron_means = zeros(1,tmp);
cnt = 1;

cover = zeros(size(I_transcript));

for g=1:length(genes),
  probe_idx = genes(g).probes(:,2);
  genes(g).hyb.I = I(probe_idx)';
    
  for i=1:length(genes(g).transcripts)
    exon_idx   = find(genes(g).probes_exonic(:,i)>0);
    intron_idx = find(genes(g).probes_intronic(:,i)>0);
    
    % convert into global indices
    exon_idx   = probe_idx(exon_idx);
    intron_idx = probe_idx(intron_idx);
      
    % I_transcript in exons is the median of the exon probes
    I_tmp = I(exon_idx);
    I_tmp = I_tmp(~isnan(I_tmp));
    if ~isempty(I_tmp),
      cover(exon_idx) = cover(exon_idx) + 1;
      I_transcript(exon_idx) = median(I_tmp);
      genes(g).hyb.I_transcript(i) = median(I_tmp);
      exon_means(cnt) = mean(I_tmp);
    else
      genes(g).hyb.I_transcript(i) = nan;
    end

    % I_transcript in introns is the median of the intron probes
    I_tmp = I(intron_idx);
    I_tmp = I_tmp(~isnan(I_tmp));
    if ~isempty(I_tmp),
      cover(intron_idx) = cover(intron_idx) + 1;
      I_transcript(intron_idx) = median(I_tmp);
      intron_means(cnt) = mean(I_tmp);
    end
    
    cnt = cnt + 1;
  end
%  if mod(g,100)==0, fprintf('  %i\r',g); end
end
%fprintf('\n');

% I_transcript in intergenic regions is global intergenic median
global_intergenic_idx = find(probe_label==LABEL.intergenic);
I_tmp = I(global_intergenic_idx);
I_transcript(global_intergenic_idx) = median(I_tmp(~isnan(I_tmp)));


I_transcript(find(cover>1)) = -1;
fprintf('%2.1f%% of probes masked due to coverage > 1.\n', ...
        100*mean(cover>1));


if VERBOSE,
  global_intron_idx = find(probe_label==LABEL.intronic);
  global_exon_idx   = find(probe_label==LABEL.exonic);

  figure;
  hold on;
  plot(exon_means, intron_means, 'k.', 'MarkerSize', 1);
  xlabel('exon intensities');
  ylabel('intron intensities');
  I_tmp = I(global_intron_idx);
  I_tmp = I_tmp(~isnan(I_tmp));
  plot([0 14], [mean(I_tmp) mean(I_tmp)], 'r');
  I_tmp = I(global_intergenic_idx);
  I_tmp = I_tmp(~isnan(I_tmp));
  plot([0 14], [mean(I_tmp) mean(I_tmp)], 'k');
  %print('-depsc', '-r300', 'I_exon_intron_correlation.eps');
  
  figure;
  hold on;
  [n_exo x_exo] = hist(I(global_exon_idx), linspace(0,20,150));  
  plot(x_exo, n_exo, 'r', 'LineWidth', 2);
  [n_ino x_ino] = hist(I(global_intron_idx), linspace(0,20,150));  
  plot(x_ino, n_ino, 'g', 'LineWidth', 2);
  [n_ige x_ige] = hist(I(global_intergenic_idx), linspace(0,20,150));
  plot(x_ige, n_ige, 'b', 'LineWidth', 2);
  legend({'exon' 'intron' 'intergenic'});
  xlabel('Log_2 intensity');
  ylabel('Number of probes');
  title('Intensity histograms');
  print('-depsc', '-r300', 'I_hist.eps');
end