function [I_n, ret_P] = predict_per_quantile(P, PM_I, seq, seq_props, OPT, PM_I_transcript, method_str)

% [I_n, ret_P] = predict_per_quantile(P, PM_I, seq, seq_props, OPT, 
%                                             PM_I_transcript, method_str)
%
% Subdivides intensities into quantiles, using a separate predictor for
% each one.
%
% P -- generic input predictor struct specifying where to load the
%   trained predictors from
% PM_I -- vector of hybridization intensities to be normalized
% seq -- an array (25 x n) containing probe sequences corresponding to
%   PM_I
% seq_props -- additional sequence property features computed from
%   probe sequences
% OPT -- a struct with parameters controlling the workflow
% PM_I_transcript -- vector of idealized transcript intensities
% methodstr -- a string specifying the normaliziation method, one of
%   'Ridge' or 'SVR'
% returns normalized intensities as well as an augmented predictor struct
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008


if isfield(OPT, 'DIST_CORRECTION') && OPT.DIST_CORRECTION,
  dist = P(1).dist;
  assert(all(size(dist) == size(PM_I)));
end

d_pred = nan(size(PM_I));
for p=1:length(P),
  PREDICTOR = P(p);
  switch method_str
   case 'Ridge',
    if ~isfield(PREDICTOR, 'w'),
      tmp = load(PREDICTOR.PARAM.fn_out);
      PREDICTOR = tmp.RR;
    end    
   case 'SVR',
    if ~isfield(PREDICTOR, 'b') || ~isfield(PREDICTOR, 'alphas'),
      tmp = load(PREDICTOR.PARAM.fn_out);
      PREDCITOR = tmp.SVR;
    end
  end
  
  if p==1,
    idx = find(PM_I<=PREDICTOR.I_max);
  elseif p==length(P)
    idx = find(PREDICTOR.I_min<=PM_I);
  else
    idx = find(PREDICTOR.I_min<=PM_I & PM_I<=PREDICTOR.I_max);
  end
  
  if isequal(method_str, 'Ridge') ...
        & (~isfield(OPT, 'LOAD_RR_PRED') || ~OPT.LOAD_RR_PRED) ...
     | isequal(method_str, 'SVR') ...
        & (~isfield(OPT, 'LOAD_SVR_PRED') || ~OPT.LOAD_SVR_PRED),
    
    PREDICTOR.X_eval_1 = seq(:,idx);

    PREDICTOR.X_eval_2 = [seq_props.gc_content(:,idx); seq_props.entr(:,idx); ...
                          seq_props.hp_sc_1(:,idx); seq_props.hp_sc_2(:,idx)];

    % standardize features
    if isfield(PREDICTOR, 'mean_X_train_2') & isfield(PREDICTOR, 'std_X_train_2')
      for i=1:size(PREDICTOR.X_eval_2,1),
        PREDICTOR.X_eval_2(i,:) = (PREDICTOR.X_eval_2(i,:) - PREDICTOR.mean_X_train_2(i)) ...
            / PREDICTOR.std_X_train_2(i);
      end        
    end
    
    % add distance features
    if PREDICTOR.DIST_CORRECTION,
      PREDICTOR.dist = dist(idx);
      PREDICTOR.X_eval_3 = zeros(length(PREDICTOR.dist_PLiF_s_x), length(PREDICTOR.dist));
      assert(size(PREDICTOR.X_eval_3,2) == size(PREDICTOR.X_eval_1,2));
      for i=1:size(PREDICTOR.X_eval_3,2),
        PREDICTOR.X_eval_3(:,i) = add_frac_vec(PREDICTOR.X_eval_3(:,i), ...
                                         PREDICTOR.dist(i), PREDICTOR.dist_PLiF_s_x);
      end
    end
    
    fprintf('fold %2i, quantile %2i, using %s.\n', ...
            PREDICTOR.fold_id, PREDICTOR.quant_id, method_str);
    switch method_str
     case 'Ridge',
      PREDICTOR = predict_Ridge(PREDICTOR);
     case 'SVR',
      PREDICTOR = predict_SVR(PREDICTOR);
    end
    
    if exist('PM_I_transcript', 'var'),
      % for evaluation (not for prediction) exclude sites 
      % where I_transcript is -1 (to to ambiguities)
      eval_idx = find(PM_I_transcript(idx) ~= -1);
      Y_pred_eval = PREDICTOR.Y_pred_eval(eval_idx);
      eval_idx = idx(eval_idx);

      % predict PM intensity difference
      PREDICTOR.Y_eval = PM_I(eval_idx) - PM_I_transcript(eval_idx);
      PREDICTOR.Q1_VL = mean(abs(Y_pred_eval - PREDICTOR.Y_eval)) ...
          / mean(abs(PREDICTOR.Y_eval - median(PREDICTOR.Y_eval)));
      PREDICTOR.Q2_VL = mean((Y_pred_eval - PREDICTOR.Y_eval).^2) ...
          / mean((PREDICTOR.Y_eval - mean(PREDICTOR.Y_eval)).^2);
    end
    if OPT.VERBOSE, disp(PREDICTOR); end
    if OPT.SAVE_PREDICTORS
      switch method_str
       case 'Ridge',
        RR = PREDICTOR;
        save(PREDICTOR.PARAM.fn_out, 'RR');
       case 'SVR',
        SVM = PREDICTOR;
        save(PREDICTOR.PARAM.fn_out, 'SVM');
      end
    end
  else
    assert(isfield(PREDICTOR, 'Y_pred_eval'));
    if OPT.VERBOSE,
      % just for a more convenient display
      tmp = PREDICTOR;
      tmp = rmfield(tmp, 'fold_id');
      tmp = rmfield(tmp, 'train_subsets');
      tmp = rmfield(tmp, 'train_probe_idx');
      tmp = rmfield(tmp, 'train_select_idx');
      tmp = rmfield(tmp, 'vald_subsets');
      tmp = rmfield(tmp, 'vald_probe_idx');
      tmp = rmfield(tmp, 'test_subsets');
      tmp = rmfield(tmp, 'test_probe_idx');
      tmp = rmfield(tmp, 'DIST_CORRECTION');
      tmp = rmfield(tmp, 'mean_X_train_2');
      tmp = rmfield(tmp, 'std_X_train_2');
      tmp = rmfield(tmp, 'PARAM');
      
      if isfield(tmp, 'w'), tmp = rmfield(tmp, 'w'); end
      if isfield(tmp, 'b'), tmp = rmfield(tmp, 'b'); end
      if isfield(tmp, 'alphas'), tmp = rmfield(tmp, 'alphas'); end
      
      if isfield(tmp, 'X_train_1'), tmp = rmfield(tmp, 'X_train_1'); end
      if isfield(tmp, 'X_train_2'), tmp = rmfield(tmp, 'X_train_2'); end
      if isfield(tmp, 'X_train_num'), tmp = rmfield(tmp, 'X_train_num'); end
      if isfield(tmp, 'Y_train'), tmp = rmfield(tmp, 'Y_train'); end
      if isfield(tmp, 'X_eval_1'), tmp = rmfield(tmp, 'X_eval_1'); end
      if isfield(tmp, 'X_eval_2'), tmp = rmfield(tmp, 'X_eval_2'); end
      if isfield(tmp, 'X_eval_num'), tmp = rmfield(tmp, 'X_eval_num'); end
      if isfield(tmp, 'X_test_1'), tmp = rmfield(tmp, 'X_test_1'); end
      if isfield(tmp, 'X_test_2'), tmp = rmfield(tmp, 'X_test_2'); end
      if isfield(tmp, 'X_test_num'), tmp = rmfield(tmp, 'X_test_num'); end
      if isfield(tmp, 'Y_eval'), tmp = rmfield(tmp, 'Y_eval'); end
      if isfield(tmp, 'Y_pred_eval'), tmp = rmfield(tmp, 'Y_pred_eval'); end
      if isfield(tmp, 'Y_test'), tmp = rmfield(tmp, 'Y_test'); end
      if isfield(tmp, 'Y_pred_test'), tmp = rmfield(tmp, 'Y_pred_test'); end
      disp(tmp);
      clear tmp;
    end
  end
  
  assert(~any(isnan(PREDICTOR.Y_pred_eval)));
  d_pred(idx) = PREDICTOR.Y_pred_eval;
  
  if exist('PM_I_transcript', 'var'),
    % for evaluation (not for prediction) exclude sites 
    % where I_transcript is -1 (to to ambiguities)
    eval_idx = find(PM_I_transcript(idx) ~= -1);
    eval_idx = idx(eval_idx);

    PREDICTOR.T1 = mean(abs(PM_I(eval_idx) - d_pred(eval_idx) - PM_I_transcript(eval_idx))) ...
        / mean(abs(PM_I(eval_idx) - PM_I_transcript(eval_idx)));
    PREDICTOR.T2 = mean((PM_I(eval_idx) - d_pred(eval_idx) - PM_I_transcript(eval_idx)).^2) ...
        / mean((PM_I(eval_idx) - PM_I_transcript(eval_idx)).^2);
  end
  
  ret_P(p) = PREDICTOR;
end

% assert that every site was assigned to at least 1 predictor
assert(~(any(isnan(d_pred) & ~isnan(PM_I))));

% transform differences to normalized intensities
I_n = PM_I - d_pred;
