function ret_P = train_per_quantile(P, PM_I, PM_I_transcript, seq, seq_props, OPT, method_str)

% ret_P = train_per_quantile(P, PM_I, PM_I_transcript, seq, seq_props,
%                                                       OPT, method_str)
%
% Splits intensity data into intensity quantiles for each of which a
% predictor will be trained.
%
% P -- a struct containing predictor info
% PM_I -- an array (1 x n) containing tiling array PM intensities (log_2
%   transformed) 
% PM_I_transcript -- an array (1 x n) containing transcript intensity
%   values over exon/intron probes of the same gene (or a global value
%   for intergenic probes)
% seq -- an array (25 x n) containing probe sequences
% seq_props -- an array (X x n) containing probe properties (e.g. GC
%   content) 
% OPT -- a struct of settings (see main_training)
% method_str -- either 'Ridge' or 'SVR' used to select the regression
%   method
% returns a predictor struct (augmented P argument)
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

% split data according to intensity quantiles
predictor_idx = split_data(PM_I, OPT.NUM_PREDICTORS);
assert(all(unique(predictor_idx) == [1:OPT.NUM_PREDICTORS]));

if P.DIST_CORRECTION,
  dist = P.dist;
end

% train predictors independently
JOB_INFO = [];
for p=1:OPT.NUM_PREDICTORS,
  idx = find(predictor_idx==p);
  idx = idx(randperm(length(idx)));
  
  P.quant_id = p;
  if p==1,
    P.I_min = -inf;
    P.I_max = max(PM_I(idx));
  elseif p==OPT.NUM_PREDICTORS,
    P.I_min = ret_P(p-1).I_max;
    P.I_max = inf;
  else
    P.I_min = ret_P(p-1).I_max;
    P.I_max = max(PM_I(idx));
  end
  
  % As a first set of features use probe sequences
  P.X_train_1 = seq(:,idx);
  
  % As a second set of features use sequence properties 
  % such as GC content, entropy and heuristic hairpin scores
  P.X_train_2 = [seq_props.gc_content(:,idx); seq_props.entr(:,idx); ...
                 seq_props.hp_sc_1(:,idx); seq_props.hp_sc_2(:,idx)];
  % standardize features
  P.mean_X_train_2 = mean(P.X_train_2,2);
  P.std_X_train_2  = std(P.X_train_2,0,2);
  for i=1:size(P.X_train_2,1),
    P.X_train_2(i,:) = (P.X_train_2(i,:) - P.mean_X_train_2(i)) ...
        / P.std_X_train_2(i);
  end
  % add distance features
  if P.DIST_CORRECTION,
    P.dist = dist(idx);
    X_train_3 = zeros(length(P.dist_PLiF_s_x), length(P.dist));
    assert(size(X_train_3,2) == size(P.X_train_1,2));
    for i=1:size(X_train_3,2),
      X_train_3(:,i) = add_frac_vec(X_train_3(:,i), ...
                                    P.dist(i), P.dist_PLiF_s_x);
    end
    P.X_train_2 = [P.X_train_2; X_train_3];
  end
  
  % predict difference between observed log_2 PM intensities and
  % constant transcript intensities
  P.Y_train = PM_I(idx) - PM_I_transcript(idx);

  switch method_str,
   case 'Ridge',
    if OPT.USE_RPROC,
      [ret_P JOB_INFO] = model_sel_Ridge(P, OPT, JOB_INFO);    
    else
      ret_P = model_sel_Ridge(P, OPT);    
    end
   case 'SVR',
    if OPT.USE_RPROC,
      [ret_P JOB_INFO] = model_sel_SVR(P, OPT, JOB_INFO);
    else
      ret_P = model_sel_SVR(P, OPT);
    end
  end
end
